@@ -57,10 +57,6 @@
 #if defined(_MSC_VER) || defined(__MINGW32__)
 #include <winsock2.h>
 #define gettimeofday(p1,p2)
-#define HAVE_TIME_H
-#include <time.h>
-#define HAVE_STDARG_H
-#include <stdarg.h>
 #define snprintf _snprintf
 #endif /* _MS_VER */
 #else /* WIN32 */
@@ -82,12 +78,14 @@ static int dumpextensions = 0;
 static int novalid = 0;
 static int nodtdattr = 0;
 static int noout = 0;
+static int nodict = 0;
 #ifdef LIBXML_HTML_ENABLED
 static int html = 0;
 #endif
 static int load_trace = 0;
 #ifdef LIBXML_XINCLUDE_ENABLED
 static int xinclude = 0;
+static int xincludestyle = 0;
 #endif
 static int profile = 0;
 
@@ -394,6 +392,10 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur, const char *filename) {
 	if (ctxt == NULL)
 	    return;
 	xsltSetCtxtParseOptions(ctxt, options);
+#ifdef LIBXML_XINCLUDE_ENABLED
+	if (xinclude)
+	    ctxt->xinclude = 1;
+#endif
 	if (profile) {
 	    res = xsltApplyStylesheetUser(cur, doc, params, NULL,
 		                          stderr, ctxt);
@@ -454,18 +456,20 @@ xsltProcess(xmlDocPtr doc, xsltStylesheetPtr cur, const char *filename) {
 
 	xmlFreeDoc(res);
     } else {
-
+        int ret;
 	ctxt = xsltNewTransformContext(cur, doc);
 	if (ctxt == NULL)
 	    return;
 	if (profile) {
-	    xsltRunStylesheetUser(cur, doc, params, output,
+	    ret = xsltRunStylesheetUser(cur, doc, params, output,
 		                        NULL, NULL, stderr, ctxt);
 	} else {
-	    xsltRunStylesheetUser(cur, doc, params, output,
+	    ret = xsltRunStylesheetUser(cur, doc, params, output,
 		                        NULL, NULL, NULL, ctxt);
 	}
-	if (ctxt->state == XSLT_STATE_ERROR)
+	if (ret == -1)
+	    errorno = 11;
+	else if (ctxt->state == XSLT_STATE_ERROR)
 	    errorno = 9;
 	else if (ctxt->state == XSLT_STATE_STOPPED)
 	    errorno = 10;
@@ -512,7 +516,8 @@ static void usage(const char *name) {
     printf("\t         file:///etc/xml/catalog are activated by default\n");
 #endif
 #ifdef LIBXML_XINCLUDE_ENABLED
-    printf("\t--xinclude : do XInclude processing on document intput\n");
+    printf("\t--xinclude : do XInclude processing on document input\n");
+    printf("\t--xincludestyle : do XInclude processing on stylesheets\n");
 #endif
     printf("\t--load-trace : print trace of all external entites loaded\n");
     printf("\t--profile or --norman : dump profiling informations \n");
@@ -604,6 +609,9 @@ main(int argc, char **argv)
         } else if ((!strcmp(argv[i], "-profile")) ||
                    (!strcmp(argv[i], "--profile"))) {
             profile++;
+        } else if ((!strcmp(argv[i], "-nodict")) ||
+                   (!strcmp(argv[i], "--nodict"))) {
+            nodict++;
         } else if ((!strcmp(argv[i], "-norman")) ||
                    (!strcmp(argv[i], "--norman"))) {
             profile++;
@@ -648,6 +656,9 @@ main(int argc, char **argv)
         } else if ((!strcmp(argv[i], "-xinclude")) ||
                    (!strcmp(argv[i], "--xinclude"))) {
             xinclude++;
+        } else if ((!strcmp(argv[i], "-xincludestyle")) ||
+                   (!strcmp(argv[i], "--xincludestyle"))) {
+            xincludestyle++;
             xsltSetXIncludeDefault(1);
 #endif
         } else if ((!strcmp(argv[i], "-load-trace")) ||
@@ -725,6 +736,8 @@ main(int argc, char **argv)
 	options = XML_PARSE_NOENT | XML_PARSE_NOCDATA;
     else if (nodtdattr)
         options = XML_PARSE_NOENT | XML_PARSE_DTDLOAD | XML_PARSE_NOCDATA;
+    if (nodict != 0)
+        options |= XML_PARSE_NODICT;
 
     /*
      * Register the EXSLT extensions and the test module
@@ -773,6 +786,20 @@ main(int argc, char **argv)
 	    style = xmlReadFile((const char *) argv[i], NULL, options);
             if (timing) 
 		endTimer("Parsing stylesheet %s", argv[i]);
+	    if (xincludestyle) {
+		if (style != NULL) {
+		    if (timing)
+			startTimer();
+#if LIBXML_VERSION >= 20603
+		    xmlXIncludeProcessFlags(style, XSLT_PARSE_OPTIONS);
+#else
+		    xmlXIncludeProcess(style);
+#endif
+		    if (timing) {
+			endTimer("XInclude processing %s", argv[i]);
+		    }
+		}
+	    }
 	    if (style == NULL) {
 		fprintf(stderr,  "cannot parse %s\n", argv[i]);
 		cur = NULL;
